from __future__ import absolute_import, print_function, unicode_literals

from itertools import islice
from tqdm.notebook import tqdm
from re import sub
import torch
from .stop_keywords import StopKeywords

from sklearn.feature_extraction.text import CountVectorizer
from numpy import array, log

class TfIdfKeywords:
    def __init__(self, use_stop_kw=False):
        self.model = self.create_model()
        self.use_stop_kw = use_stop_kw
        if self.use_stop_kw:
            self.kws = self.create_keywords()

    def create_model(self):
        vectorizer = CountVectorizer()
        return vectorizer

    def create_keywords(self):
        stop_obj = StopKeywords("stop_keywords.csv")
        stop_kws = stop_obj.get_keywords()

        return stop_kws

    def predict(self, text):
        tf = self.model.fit_transform([text.lower()])
        tf = tf.toarray()
        tf = log(tf + 1)
        tfidf = tf.copy()
        words = array(self.model.get_feature_names())
        if self.use_stop_kw:
            for k in tqdm(self.kws.keys()):
                if k in words:
                    tfidf[:, words == k] = tfidf[:, words == k] * self.kws[k]

        for j in range(tfidf.shape[0]):
            keywords = words[tfidf[j, :].argsort()[-5:][::-1]]
        return keywords

if __name__=="__main__":
    obj = TfIdfKeywords(use_stop_kw=True)
    res = obj.predict('The Free Software Foundation (FSF) is a nonprofit with a worldwide mission to promote computer user freedom. Escape to Freedom now also available in Mandarin and Spanish "Escape to Freedom" is a new animated video from the Free Software Foundation (FSF), giving an introduction to the concepts behind software freedom: both what we gain by having it, and what rights are at stake. We now have the video available in Mandarin and Spanish language tracks. 2022 Bulletin: "Unjust Algorithms" by Zoë Kooyman Developments in artificial intelligence (AI) injustices have rapidly taken a turn for the worse in recent years. Algorithmic decision-making systems are used more than ever by organizations, educational institutions, and governments looking for ways to increase understanding and make predictions. The Free Software Foundation (FSF) is working through this issue, and its many scenarios, to be able to say useful things about how this relates to software freedom. Our call for papers on Copilot was a first step in this direction. Free software means that the users have the freedom to run, edit, contribute to, and share the software. Thus, free software is a matter of liberty, not price. We have been defending the rights of all software users for the past 35 years. Help sustain us for many more; become an associate member today. Subscribe to our monthly newsletter, the Free Software Supporter: Our initiatives Defective by Design is a grassroots campaign to eliminate Digital Restrictions Management (DRM) in media and devices. Read the Email Self-Defense Guide to get started with email encryption, a skill necessary to combat bulk surveillance. The End Software Patents initiative fights to abolish software patents around the world. Join us in calling for a Web that respects our freedom by being compatible with free software and stand up against nonfree JavaScript. The Free Software Directory is a collaborative catalog of computer programs and apps that are fully free. The GNU operating system is a continuously evolving, complete operating system made entirely of free software. LibrePlanet is our global network of free software activism, including events like our annual conference, and online collaboration spaces. The Licensing and Compliance Lab is the preeminent resource for public education on licensing best practices and enforcing the GPL. The "Respects Your Freedom" program certifies retailers who sell hardware in a manner that respects the rights of their users. "This community that we have, that were building, that does so much, has to grow. We cant compete with Apple, we cant compete with Google, directly, in the field of resources. What we can eventually do is head count and heart count. We can compete on the ground of ideology because ours is better." -- Edward Snowden, NSA whisteblower, speaking at LibrePlanet 2016. Take Action Bulletin Old but not forgotten Unjust Algorithms Verifying free software: The basics The need for free software education now Overcoming the hurdle of "industry standard" in education technology Read the current issue of the Bulletin and check out the archives. Shop Beat the summer heat with a stylish GNU baseball cap!')
    print(res)



from __future__ import absolute_import, print_function, unicode_literals

from tqdm.notebook import tqdm
from itertools import islice
from re import sub


class StopKeywords:
    def __init__(self, stop_kw_file="stop_keywords.csv"):
        self.stop_kw_file = stop_kw_file

    def get_keywords(self):
        num_lines = sum(1 for line in open(self.stop_kw_file))
        with open(self.stop_kw_file) as kwfile:
            dict_idf = {}
            with tqdm(total=num_lines) as pbar:
                for i, line in tqdm(islice(enumerate(kwfile), 1, None)):
                    try:
                        cells = line.split(",")
                        idf = float(sub("[^0-9.]", "", cells[3]))
                        dict_idf[cells[0]] = idf
                    except Exception as e:
                        print(i, e)
                        print("Error on: " + line)
                        continue
                    finally:
                        pbar.update(1)


        return dict_idf


if __name__=="__main__":
    stop_kw_obj = StopKeywords(stop_kw_file="stop_keywords.csv")
    stop_kw_obj.read_lines()



from __future__ import absolute_import, print_function, unicode_literals

# TextRank抽取关键词
from summa import keywords

class TextRankKeywords:
    def __init__(self):
        self.model = self.create_model()

    def create_model(self):
        pass

    def predict(self, text):
        kws = (keywords.keywords(text, words=5)).split("\n")
        return kws



"""Main module."""
from keywords_en.utils import TfIdfKeywords
from keywords_en.utils import TextRankKeywords 
from keywords_en.utils import TopicRankKeywords 
from keywords_en.utils import YakeKeywords 
from keywords_en.utils import KeyBertKeywords 

class KeyWordsEn:
    def __init__(self, algo_name="tfIdf", text=""):
        self.text = text
        self.algo_method = getattr(self, f'kw_{algo_name}', None)
        if not self.algo_method:
            raise NotImplementedError

        self.keywords = self.algo_method()

    def kw_tfIdf(self):
        kw_obj = TfIdfKeywords(use_stop_kw=False)
        kw_res = kw_obj.predict(self.text)
        return kw_res

    def kw_textRank(self):
        kw_obj = TextRankKeywords()
        kw_res = kw_obj.predict(self.text)
        return kw_res
         
    def kw_topicRank(self):
        kw_obj = TopicRankKeywords()
        kw_res = kw_obj.predict(self.text)
        return kw_res

    def kw_yake(self):
        kw_obj = YakeKeywords()
        kw_res = kw_obj.predict(self.text)
        return kw_res

    def kw_keyBert(self):
        kw_obj = KeyBertKeywords()
        kw_res = kw_obj.predict(self.text)
        return kw_res



#!/usr/bin/env python

"""The setup script."""

from setuptools import setup, find_packages

with open('README.md') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = open("requirements.txt").readlines()

test_requirements = ['pytest>=3', ]

setup(
    author="nilecui",
    author_email='nilecui@gmail.com',
    python_requires='>=3.6',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    description="keywords extract",
    entry_points={
        'console_scripts': [
            'keywords_en=keywords_en.cli:main',
        ],
    },
    install_requires=requirements,
    license="GNU General Public License v3",
    long_description=readme,
    long_description_content_type="text/markdown",  # 长描述内容类型
    include_package_data=True,
    keywords='keywords_en',
    name='keywords_en',
    packages=find_packages(include=['keywords_en', 'keywords_en.*']),
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/nilecui/keywords_en',
    version='0.1.3',
    zip_safe=False,
)



from __future__ import absolute_import, print_function, unicode_literals
from .tf_idf_kw import TfIdfKeywords
from .text_rank_kw import TextRankKeywords
from .topic_rank_kw import TopicRankKeywords
from .yake_kw import YakeKeywords
from .key_bert_kw import KeyBertKeywords
